(*  Title:      src/Tools/isac/BridgeJEdit/VSCode_Example.thy
    Author:     Walther Neuper, JKU Linz
    (c) due to copyright terms

Runnig example for developing Isabelle/Isac via Isabelle/PIDE in BridgeJEdit.
File will go to test/Tools/isac/BridgeJEdit/VSCode_Example.thy
compare         test/Tools/isac/BridgeJEdit/vscode-example.sml
*)

theory VSCode_Example 
  imports Calculation
begin

section \<open>Boilerplate, the example for Isabelle Workshop 2022\<close>

subsection \<open>Specification Phase\<close>
text \<open>
  The \<open>Specification\<close> has a fixed format; the user shall be given a template,
  See notes in src/../template.sml.

  Below there are different versions of Example "Diff_App-No.123a" for reference;
  the versions refer to different situations in educational settings.

  The actual development is done in test/../Test_VSCode_Example.thy on \<open>Example_TEST\<close>.
  According to successful results of development the versions below will be updated.
\<close>

subsubsection \<open>Complete Specification\<close>
text \<open>
* This is one correct result of interactive Specification.
* Or this might be presented to the student in one go in order to start Solution immediately
  (and nevertheless make the Specification explicit)
Note that \<open>Problem "univariate_calculus/Optimisation"\<close> and 
\<open>Problem_Ref: "univariate_calculus/Optimisation"\<close> are redundant; the latter is for input,
the former is given initially and needs to be updated in accordance to \<open>Problem_Ref\<close>.
\<close>

Example "Diff_App-No.123a"
  Specification:
    Model:
      Given: \<open>Constants r = 7\<close>
      Where: \<open>0 < r\<close>
      Find:  \<open>Maximum A\<close> \<open>AdditionalValues u, v\<close>
      Relate: \<open>Extremum A = 2 \<sqdot> u \<sqdot> v − u \<up> 2\<close> \<open>SideCondition (u / 2) \<up> 2 + (2 / v) \<up> 2 = r \<up> 2\<close>
    References:
      Theory_Ref: "Diff_App"
  (*\<Otimes>*) Problem_Ref: "univariate_calculus/Optimisation"
  (*\<Odot>*) Method_Ref: "Optimisation/by_univariate_calculus"
(*Solution:*)

subsubsection \<open>Empty Specification\<close>
text \<open>
  This is presented to the student in one go in order to start interactive Specification.
\<close>
text \<open> (*.. remove as soon as this works as Isabelle/Isar commands ..*)

Example "Diff_App-No.123a"
  Specification:
    Model:
      Given: \<open>Constants _\<close>
      Where: \<open>0 < r\<close>
      Find:  \<open>Maximum _\<close> \<open>AdditionalValues _\<close>
      Relate: \<open>Extremum _\<close> \<open>SideCondition _\<close>     References:
    References:
      Theory_Ref: "Diff_App"
  (*\<Otimes>*) Problem_Ref: "univariate_calculus/Optimisation"
  (*\<Odot>*) Method_Ref: "Optimisation/by_univariate_calculus"
(*Solution:*)

       (* remove as soon as this ^^^ works as Isabelle/Isar commands ..*) \<close>

subsubsection \<open>Plain Example\<close>
text \<open>
  This can occur in between text offering to start an interactive Calculation by using
  "Diff_App-No.123a" as a link.
\<close>
text \<open> (*.. remove as soon as this works as Isabelle/Isar commands ..*)

Example "Diff_App-No.123a"

       (* remove as soon as this ^^^ works as Isabelle/Isar commands ..*) \<close>

subsubsection \<open>Immediate Start of Interactive Solving\<close>
text \<open>
  This is requested by teachers in case of CAS_Cmd, e.g. "solve (x+1=2, x)"
  where Specification usually is not interesting to students
  and thus done automatically by ISAC in the background.
  All field tests so far were done with skipping Specification;
  however, the keyword Specification remains visible as a reminder
\<close>
text \<open> (*.. remove as soon as this works as Isabelle/Isar commands ..*)

Example "Diff_App-No.123a"
  Problem "univariate_calculus/Optimisation"
    Specification:
    Solution:

       (* remove as soon as this ^^^ works as Isabelle/Isar commands ..*) \<close>

subsubsection \<open>Show the Model only, an intermediate mode\<close>
text \<open>
  While teachers a high-school like to skip Specification completely,
  many academic lecturers are aware of the importance of clarifying "what" the problem is
  before clarifying "how" a problem can be solved.
  This mode of interaction is an intermediate step between high-school and academic engineering.
\<close>
text \<open> (*.. remove as soon as this works as Isabelle/Isar commands ..*)

Example "Diff_App-No.123a"
  Problem "univariate_calculus/Optimisation"
    Specification:
      Model:
        Given: \<open>Constants r = 7\<close>
        Where: \<open>0 < r\<close>
        Find:  \<open>Maximum A\<close> \<open>AdditionalValues u, v\<close>
        Relate: \<open>Extremum A = 2 \<sqdot> u \<sqdot> v − u \<up> 2\<close> \<open>SideCondition (u / 2) \<up> 2 + (2 / v) \<up> 2 = r \<up> 2\<close>
      References:
    Solution:

       (* remove as soon as this ^^^ works as Isabelle/Isar commands ..*) \<close>

subsubsection \<open>Show the Model associated to Method_Ref\<close>
text \<open>
  The Model of a method usually comprises more items than the model of a Problem: all these
  are required to run the program such that it automatically generates a Solution. For instance,
  compare \<open>problem pbl_bieg\<close> and \<open>method met_biege_2\<close> in \<open>Biegelinie.thy\<close>
  (TODO: method/program for \<open>Problem "univariate_calculus/Optimisation"\<close>)
  The Model of a method is usually called a guard.

  Note: the method is not yet implemented, so the Model is spurious.
\<close>
text \<open> (*.. remove as soon as this works as Isabelle/Isar commands ..*)

Example "Diff_App-No.123a"
  Problem "univariate_calculus/Optimisation"
    Specification:
      Model:
        Given: \<open>Maximum A\<close> \<open>Constants r = 7\<close> \<open>differentiateFor x\<close>
          \<open>Domain {0 <..< r}\<close> \<open>errorBound eps = 0.01\<close>
        Where:
        Find: \<open>extrema\<close>
        Relate: 
    References:
      Theory_Ref: "Diff_App"
  (*\<Otimes>*) Problem_Ref: "univariate_calculus/Optimisation"
  (*\<Odot>*) Method_Ref: "Optimisation/by_univariate_calculus"
(*Solution:*)

       (* remove as soon as this ^^^ works as Isabelle/Isar commands ..*) \<close>


section \<open>Stepwise Development\<close>

subsection \<open>preparing VSCode_Example\<close>
ML \<open>
(**** preparing VSCode_Example ########################################################### ****)
"----------- preparing VSCode_Example ----------------------------------------------------------";
"----------- preparing VSCode_Example ----------------------------------------------------------";
\<close> text \<open>
--- maximum example with Step.specify_do_next --- from test/../step.sml
val model = [
(*Problem model:*)
  "fixedValues [r=Arbfix]", "maximum A",
  "valuesFor [a,b::real]",
  "relations [A=a*(b::real), (a/2) \<up> 2 + (b/2) \<up> 2 = (r::real) \<up> 2]",
  "relations [A=a*(b::real), (a/2) \<up> 2 + (b/2) \<up> 2 = (r::real) \<up> 2]",
  "relations [A=a*(b::real), a/2=r*sin alpha, b/2 = (r::real)*cos (alpha::real)]",
(*MethodC model:*)
  "boundVariable a", "boundVariable b", "boundVariable alpha",
  "interval {x::real. 0 <= x & x <= 2*r}",
  "interval {x::real. 0 <= x & x <= 2*r}",
  "interval {x::real. 0 <= x & x <= pi}",
  "errorBound (eps=(0::real))"];
val (dI',pI',mI') = 
("Diff_App", ["maximum_of", "function"], ["Diff_App", "max_by_calculus"]);
\<close> text \<open>
from paper "Towards Accessible Formal Mathematics with ISAC and Isabelle/VSCode"

F_I \<equiv> [ [r = 7], [A, [u, v]], [A = 2 * u * v - u \<up> 2 , ( 2 / u ) \<up> 2 + ( 2 / v ) \<up> 2 = r \<up> 2], {0 <..< r} ]
--- type conflict ^^ ----------^^
F_II \<equiv> [ [r = 7], [A, \<alpha>], [A = 2 * u * v - u \<up> 2 , u / 2 = r * sin \<alpha>, 2 / v = r * cos \<alpha>], {0 <..< \<pi> / 2} ]
\<close> ML \<open>
val model = [
(*Problem model:*)
  "Constants [r = (7::real)]", "Maximum A", "AdditionalValues [u, v]",
  "Extremum (A = 2 * u * v - u \<up> 2)",
  "SideConditions [((u::real) / 2) \<up> 2 + (2 / v) \<up> 2 = r \<up> 2]",
  "SideConditions [((u::real) / 2) \<up> 2 + (2 / v) \<up> 2 = r \<up> 2]",
  "SideConditions [(u::real) / 2 = r * sin \<alpha>, 2 / v = r * cos \<alpha>]",
(*MethodC model:*)
  "FunctionVariable a", "FunctionVariable b", "FunctionVariable \<alpha>",
  "Domain {0 <..< r}",
  "Domain {0 <..< r}",
  "Domain {0 <..< \<pi> / 2}",
  "ErrorBound (\<epsilon> = (0::real))"
]: TermC.as_string list;
val refs = 
("Diff_App", ["univariate_calculus", "Optimisation"], ["Optimisation", "by_univariate_calculus"]);
\<close> ML \<open>
\<close>

subsection \<open>Specification Phase\<close>
text \<open> 
  Stepwise development of \<open>Outer_Syntax.command \<^command_keyword>\<open>Example\<close>\<close> begins with 
  the changeset https://hg.risc.uni-linz.ac.at/wneuper/isa/rev/9d98791b4080
  in the repository.

  The intermediate steps below will be deleted as soon as all above representations 
  of Example "Diff_App-No.123a" work out.
\<close>

subsubsection \<open>Complete Specification at once\<close>
Example "Diff_App-No.123a"
  Specification:
    Model:
      Given: \<open>Constants [r = 7]\<close>
      Where: \<open>0 < r\<close>
      Find:  \<open>Maximum A\<close> \<open>AdditionalValues [u, v]\<close>
      Relate: \<open>Extremum (A = 2 * u * v - u \<up> 2)\<close> 
        \<open>SideConditions [((u::real) / 2) \<up> 2 + (2 / v) \<up> 2 = r \<up> 2]\<close>
    References:
      Theory_Ref: "Diff_App"
  (*\<Otimes>*) Problem_Ref: "univariate_calculus/Optimisation"
  (*\<Odot>*) Method_Ref: "Optimisation/by_univariate_calculus"
(*Solution:*)

ML \<open>
val state = the_data @{theory}; (* state filled from the above Example by \<open>fun update_state\<close> *)
\<close> text \<open>
val (o_model, refs, _) = Ctree.get_obj Ctree.g_origin state []
val problem_model = Ctree.get_obj Ctree.g_pbl state []
val method_model = Ctree.get_obj Ctree.g_met state []
\<close> text \<open> (* this was with Specify.finish_phasePIDE in init_ctree, cf. 28da4f69d32d *)
if length o_model = 12  andalso refs = 
  ("Diff_App", ["univariate_calculus", "Optimisation"], ["Optimisation", "by_univariate_calculus"])
  andalso length problem_model = 5 andalso length method_model = 8
then () else error "initialised state CHANGED";
\<close> text \<open>
if length o_model = 12  andalso refs = 
  ("Diff_App", ["univariate_calculus", "Optimisation"], ["Optimisation", "by_univariate_calculus"])
  andalso length problem_model = 0 andalso length method_model = 0
then () else error "initialised state CHANGED";
\<close> ML \<open>
\<close>

subsubsection \<open>Specification step by step\<close>
text \<open>
  see $ISABELLE_ISAC_TEST/Tools/isac/BridgeJEdit/Test_VSCode_Example.thy
\<close> ML \<open>
\<close> ML \<open>
\<close> ML \<open>
\<close>


end



