(* Title:  Specify/specify-step.sml
   Author: Walther Neuper
   (c) due to copyright terms
                                                                          
Code for the specify-phase in analogy to structure Solve_Step for the solve-phase.
*)

signature SPECIFY_STEP =
sig
  val check: Tactic.input -> Calc.T -> Applicable.T
  val add: Tactic.T -> Istate_Def.T * Proof.context -> Calc.T -> Test_Out.T

  val complete_for: MethodC.id -> Calc.T -> O_Model.T * Proof.context * I_Model.T
end

(**)
structure Specify_Step(**): SPECIFY_STEP(**) =
struct
(**)

fun complete_for mID (ctree, pos) =
  let
    val {origin = (o_model, _, _), probl = i_prob, ctxt,
       ...} = Calc.specify_data (ctree, pos);
    val {model = m_patt, where_, where_rls, ...} = MethodC.from_store ctxt mID
    val {origin = (root_model, _, _), ...} = Calc.specify_data (ctree, ([], Pos.Und))
    val (o_model', ctxt') = O_Model.complete_for m_patt root_model (o_model, ctxt)
    val (_, (i_model, _)) = M_Match.match_itms_oris ctxt i_prob (m_patt, where_, where_rls) o_model';
  in
    (o_model', ctxt', i_model)
  end

(*
  check tactics (input by the user, mostly) for applicability
  and determine as much of the result of the tactic as possible initially.
*)
fun check (Tactic.Add_Find ct') _ = Applicable.Yes (Tactic.Add_Find' (ct', [(*filled later*)]))
    (*Add_.. should reject (dsc //) (see fmz=[] in sqrt*)
  | check (Tactic.Add_Given ct') _ = Applicable.Yes (Tactic.Add_Given' (ct', [(*filled later*)]))
  | check (Tactic.Add_Relation ct') _ = Applicable.Yes (Tactic.Add_Relation' (ct', [(*filled later*)]))
    (*required for corner cases, e.g. test setup in inssort.sml*)
  | check (Tactic.Del_Find ct') _ = Applicable.Yes (Tactic.Del_Find' ct')
  | check (Tactic.Del_Given ct') _ = Applicable.Yes (Tactic.Del_Given' ct')
  | check (Tactic.Del_Relation ct') _ = Applicable.Yes (Tactic.Del_Relation' ct')
(*T_TESTold* )
  | check Tactic.Model_Problem (pt, pos as (p, _)) =
      let 
        val (pI', ctxt) = case Ctree.get_obj I pt p of
          Ctree.PblObj {origin = (_, (_, pI', _), _), ctxt, ...} => (pI', ctxt)
        | _ => raise ERROR "Specify_Step.check Model_Problem: uncovered case Ctree.get_obj"
	      val {model, ...} = Problem.from_store (Ctree.get_ctxt pt pos) pI'
	      val pbl = I_Model.init model
      in Applicable.Yes (Tactic.Model_Problem' (pI', pbl, [])) end
( *T_TEST*)
  | check Tactic.Model_Problem (pt, pos as (p, _)) =
      let 
        val (o_model, pI') = case Ctree.get_obj I pt p of
          Ctree.PblObj {origin = (o_model, (_, pI', _), _), ...} => (o_model, pI')
        | _ => raise ERROR "Specify_Step.check Model_Problem: uncovered case Ctree.get_obj"
	      val {model = model_patt, ...} = Problem.from_store (Ctree.get_ctxt pt pos) pI'
	      val pbl = I_Model.init_TEST o_model model_patt
      in Applicable.Yes (Tactic.Model_Problem' (pI', I_Model.TEST_to_OLD pbl, [])) end
(*T_TESTnew*)
  | check (Tactic.Refine_Problem pI) (pt, (p, _)) =
      let
        val (dI, dI', itms, ctxt) = case Ctree.get_obj I pt p of
            Ctree.PblObj {origin = (_, (dI, _, _), _), spec = (dI', _, _), probl = itms, ctxt, ...}
              => (dI, dI', itms, ctxt)
          | _ => raise ERROR "Specify_Step.check Refine_Problem: uncovered case Ctree.get_obj"
      	val thy = if dI' = ThyC.id_empty then dI else dI';
      in
        case Refine.problem (ThyC.get_theory ctxt thy) pI itms of
          NONE => Applicable.No
            (Tactic.input_to_string ctxt (Tactic.Refine_Problem pI) ^ " not applicable")
	      | SOME (rf as (pI', _)) =>
      	   if pI' = pI
      	   then Applicable.No
      	     (Tactic.input_to_string ctxt (Tactic.Refine_Problem pI) ^ " not applicable")
      	   else Applicable.Yes (Tactic.Refine_Problem' rf)
    end
  | check (Tactic.Refine_Tacitly pI) (pt, (p, _)) =
     let 
       val (oris, ctxt) = case Ctree.get_obj I pt p of
         Ctree.PblObj {origin = (oris, _, _), ctxt, ...} => (oris, ctxt)
       | _ => raise ERROR "Specify_Step.check Refine_Tacitly: uncovered case Ctree.get_obj"
     in
       case Refine.by_o_model ctxt oris pI of
	       SOME pblID => 
	         Applicable.Yes (Tactic.Refine_Tacitly' (pI, pblID, ThyC.id_empty, MethodC.id_empty, [(*filled later*)]))
	     | NONE => Applicable.No
	         (Tactic.input_to_string ctxt (Tactic.Refine_Tacitly pI) ^ " not applicable")
     end
  | check (Tactic.Specify_Method mID) (ctree, pos) =
      let
        val (o_model, _, i_model) = complete_for mID (ctree, pos)
      in
        Applicable.Yes (Tactic.Specify_Method' (mID, o_model, i_model))
      end
  | check (Tactic.Specify_Problem pID) (pt, pos as (p, _)) =
      let
		    val (oris,  pI,  pI', itms, ctxt) = case Ctree.get_obj I pt p of
		      Ctree.PblObj {origin = (oris, (_, pI, _), _), spec = (_, pI', _), probl = itms, ctxt, ...}
		        => (oris,  pI,  pI', itms, ctxt)
        | _ => raise ERROR "Specify_Step.check Specify_Problem: uncovered case Ctree.get_obj"
        val {model, where_, where_rls, ...} = Problem.from_store (Ctree.get_ctxt pt pos) pID;
        val pbl = if pI' = Problem.id_empty andalso pI = Problem.id_empty
          then (false, (I_Model.init model, []))
          else M_Match.match_itms_oris ctxt itms (model, where_, where_rls) oris;
       in
         Applicable.Yes (Tactic.Specify_Problem' (pID, pbl))
       end
  | check (Tactic.Specify_Theory dI)_ = Applicable.Yes (Tactic.Specify_Theory' dI)
  | check Tactic.Empty_Tac _ = Applicable.No "Empty_Tac is not applicable"
  | check tac (ctree, pos) =
    let
      val ctxt = Ctree.get_ctxt ctree pos
    in
      raise ERROR ("Specify_Step.check called for " ^
        Tactic.input_to_string ctxt tac ^ " at" ^ Pos.pos'2str pos)
    end;

(* exceed line length, because result type will change *)
fun add (Tactic.Model_Problem' (_, itms, met)) (_, ctxt) (pt, pos as (p, _)) =
      let
        val pt = Ctree.update_pbl pt p itms
  	    val pt = Ctree.update_met pt p met
      in
        (pos, [], Test_Out.PpcKF (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] []), pt)
      end
  | add (Tactic.Add_Given' (_, itmlist)) (_, ctxt) (pt, pos as (p, p_)) = 
      (pos, [], Test_Out.PpcKF (Test_Out.Upblmet,P_Model.from (Proof_Context.theory_of ctxt) [][]),
         case p_ of
           Pos.Pbl => Ctree.update_pbl pt p itmlist
  	     | Pos.Met => Ctree.update_met pt p itmlist
         | _ => raise ERROR ("uncovered case " ^ Pos.pos_2str p_))
  | add (Tactic.Add_Find' (_, itmlist)) (_, ctxt) (pt, pos as (p, p_)) = 
      (pos, [], (Test_Out.PpcKF (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] [])),
         case p_ of
           Pos.Pbl => Ctree.update_pbl pt p itmlist
  	     | Pos.Met => Ctree.update_met pt p itmlist
         | _ => raise ERROR ("uncovered case " ^ Pos.pos_2str p_))
  | add (Tactic.Add_Relation' (_, itmlist)) (_, ctxt) (pt, pos as (p, p_)) = 
      (pos, [],  Test_Out.PpcKF (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] []),
         case p_ of
           Pos.Pbl => Ctree.update_pbl pt p itmlist
  	     | Pos.Met => Ctree.update_met pt p itmlist
         | _ => raise ERROR ("uncovered case " ^ Pos.pos_2str p_))
  | add (Tactic.Specify_Theory' domID) (_, ctxt) (pt, pos as (p,_)) = 
      (pos, [] , Test_Out.PpcKF  (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] []),
        Ctree.update_domID pt p domID)
  | add (Tactic.Specify_Problem' (pI, (_, (itms, _)))) (_, ctxt) (pt, (p, _)) =
      let
        val pt = Ctree.update_pbl pt p itms
        val pt = Ctree.update_pblID pt p pI
      in
        ((p, Pos.Pbl), [], Test_Out.PpcKF (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] []), pt)
      end
  | add (Tactic.Specify_Method' (mID, oris, itms)) (_, ctxt) (pt, (p, _)) =
      let
        val pt = Ctree.update_oris pt p oris
        val pt = Ctree.update_met pt p itms                            
        val pt = Ctree.update_metID pt p mID
      in
        ((p, Pos.Met), [], Test_Out.PpcKF (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] []), pt)
      end
  | add (Tactic.Refine_Tacitly' (_, pIre, domID, metID, pbl)) (_, ctxt) (pt, pos as (p, _)) = 
      let
        val pt = Ctree.update_pbl pt p pbl
        val pt = Ctree.update_orispec pt p (domID, pIre, metID)
      in
        (pos, [], Test_Out.PpcKF (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] []), pt)
      end
  | add (Tactic.Refine_Problem' (pI, _)) (_, ctxt) (pt, pos as (p, _)) =
      let
        val (dI, _, mI) = Ctree.get_obj Ctree.g_spec pt p
        val pt = Ctree.update_spec pt p (dI, pI, mI)
      in
        (pos, [], Test_Out.PpcKF (Test_Out.Upblmet, P_Model.from (Proof_Context.theory_of ctxt) [] []), pt)
      end
  | add (Tactic.Begin_Trans' t) l (pt, pos as (p, Frm)) =
      let
        val (pt, c) = Ctree.cappend_form pt p l t
        val pt = Ctree.update_branch pt p Ctree.TransitiveB
        val p = (Pos.lev_on o Pos.lev_dn (* starts with [...,0] *)) p
        val (pt, c') = Ctree.cappend_form pt p l t
        val ctxt = Ctree.get_ctxt pt pos
      in
        ((p, Frm), c @ c', Test_Out.FormKF (UnparseC.term ctxt t), pt)
      end
  | add (Tactic.End_Trans' tasm) l (pt, (p, _)) = (* used at all ? *)
      let
        val p' = Pos.lev_up p
        val (pt, c) = Ctree.append_result pt p' l tasm Ctree.Complete
      in
        ((p', Pos.Res), c, Test_Out.FormKF "DUMMY" (*term2str t ..ERROR (t) has not been declared*), pt)
      end
  | add m' (_, ctxt) (_, pos) =
      raise ERROR ("Specify_Step.add: not impl.for " ^ Tactic.string_of ctxt m' ^ " at " ^ Pos.pos'2str pos)

(**)end(**);
